#include "benchmark.h"

// time units per seconds - this is 1 msec
#define SCALING 1000.0

// time allowed for formula generation/merging in seconds
int timeout = 3600;

// timeout handler which just raises the flag
void alarm_handler(int) {
	timedOut = true;
}

Benchmark::Benchmark(void) : produced(0), gbcnum(0), usertime(0), nodesinbdd(0) {}

void Benchmark::start(bool t) {
	timed = t;
	
	// get bdd stats
	bddStat stat;
	bdd_stats(&stat);
	produced = stat.produced;
	gbcnum = stat.gbcnum;
	
	// get no of dilations
	d = dilations;
	// get no of conjunctions
	c = conjunctions;
	
	// get processor time
	usertime = clock();

	if(timed) {
		timedOut = false;
		// setup timeout timer
		signal(SIGALRM, alarm_handler);
		alarm(timeout);
	}
}

void Benchmark::terminate(void) {
	// set time to timeout
	usertime = timeout * CLOCKS_PER_SEC;
	// set dilations to maximum possible
	d = propvars;
	c = UINT_MAX;
	// set produced to maximum value possible
	produced = LONG_MAX;
	// zero out the rest as it is not used in the stats
	gbcnum = 0;
	nodesinbdd = 0;
	
	// reset flag
	timedOut = false;
}

void Benchmark::stop(const bdd & b) {
	// disable timer if still running
	alarm(0);
	
	if(!outOfMem && (!timed || !timedOut)) {
		// get processor time
		usertime = clock() - usertime;
		// get no of dilations
		d = dilations - d;
		c = conjunctions - c;
		// get bdd stats
		bddStat stat;
		bdd_stats(&stat);
		produced = stat.produced - produced;
		gbcnum = stat.gbcnum - gbcnum;
		nodesinbdd = bdd_nodecount(b);
	} else {
		terminate();
	}
}

void Benchmark::stop(const KSet & kset, const bdd & IC) {
	// disable timer if still running
	alarm(0);
	
	if(!outOfMem && (!timed || !timedOut)) {
		// get processor time
		usertime = clock() - usertime;
		// get no of dilations
		d = dilations - d;
		c = conjunctions - c;
		// get bdd stats
		bddStat stat;
		bdd_stats(&stat);
		produced = stat.produced - produced;
		gbcnum = stat.gbcnum - gbcnum;

		// count collective amount of nodes	
		bdd * table = new bdd[kset.size()+1];
		std::copy(kset.begin(), kset.end(), table);
		table[kset.size()] = IC;
		nodesinbdd = bdd_anodecount(table, kset.size());
		delete [] table;
	
	} else {
		terminate();
	}
}

std::ostream& operator <<(std::ostream &os,const Benchmark &obj) {
	os <<	obj.produced	<< ',' << 
		obj.nodesinbdd	<< ',' << 
		obj.d		<< ',' << 
		obj.c		<< ',' <<
		(long long)round(SCALING*(double)obj.usertime/CLOCKS_PER_SEC) << ',' << 
		obj.gbcnum;
	return os;
}
