#ifndef BENCHMARK_H
#define BENCHMARK_H

#include <iostream>
#include <iomanip>
#include <ctime>
#include <cmath>
#include <climits>

#include <unistd.h>
#include <signal.h>

#include "dilations.h"

// variable used to store maximum time for formula generation
// and merging (separately)
// formulae which take too long are discarded
// mergings which take too long are given a maximum value for 'produced'
// and zero to all other variables
extern int timeout;

class Benchmark {
	public:
		Benchmark(void);
		
		// argument indicate whether the timer should be used
		void start(bool timed);
		void stop(const bdd & b);
		void stop(const KSet & kset, const bdd & IC);
		
		friend std::ostream& operator<<(std::ostream &os, const Benchmark &obj);
	
	private:
		// number of BDD nodes produced
		long produced;
		// number of garbage collections occurred
		int gbcnum;
		// milliseconds of usertime spent
		//float usertime;
		clock_t usertime;
		// nodes in bdd produced
		int nodesinbdd;
		// number of dilations of all KBs 
		unsigned d;
		// number of conjunctions tested
		unsigned c;
		// is the run being watched for timeout
		bool timed;
		
		void terminate(void);
};

std::ostream& operator <<(std::ostream &os,const Benchmark &obj);

#endif
