#ifndef DILATIONS_H
#define DILATIONS_H

#include <iostream>
#include <vector>
#include <numeric>
#include <bdd.h>
#include <cassert>

// knowledge profile/set type
typedef std::vector<bdd> KSet;
// set of profiles
typedef std::vector<KSet> KSetSet;
// vector of numbers
typedef std::vector<unsigned> IVec;
// type of merging operator
typedef bdd (*MergeOperator)(const KSet &, const bdd &); 

// number of propositional variables visible to the user
extern unsigned propvars;
// number of dilations (per profile, not per formula) performed up to this point
extern unsigned dilations;
// how many conjunctions have been tested for?
extern unsigned conjunctions;

class Signature {
public:
	// propositional or first-order printing mode
	bool first_order;
	// first-order domain size 
	unsigned dsize;
	// first-order predicate arity
	unsigned arity;
	// number of first-order predicates
	unsigned predicates;

	Signature() :
		first_order(false),
		dsize(1),
		arity(1),
		predicates(2)
		{}
};

extern Signature signature;

// // propositional or first-order printing mode
// extern bool first_order;
// // first-order domain size 
// extern unsigned dsize;
// // first-order predicate arity
// extern unsigned arity;
// // number of first-order predicates
// extern unsigned predicates;

// boolean flag for termination after timer expiration
extern volatile bool timedOut;
// boolean flag for termination after memory exhaustion
extern volatile bool outOfMem;

// macro for general termination condition
#define TERMINATE() (timedOut||outOfMem)

// sums up vector
#define vsum(v) (std::accumulate((v).begin(), (v).end(), 0))

// vector max
#define vmax(v) (*std::max_element((v).begin(), (v).end()))

// initialise seed from current time
unsigned getSeed(void);
// handler prints models of a bdd
void allsatPrintHandler(char *varset, int size);
// initialise bdd package and auxiliary variables
void init(void);
// do pre-termination cleanup wrt the bdd package
void cleanup(void);
// dilate a formula
bdd dilate(const bdd &);
// // initiate variable reordering if free space is low
// void check_reorder(void);

// returns the bdd for the conjunction of all bdds in kset, with integrity constraints IC
inline bdd conjunction(const KSet &k, const bdd &IC) { 
	bdd conj = IC;
	for(KSet::const_iterator ptr=k.begin(); conj!=bddfalse && ptr!=k.end(); ++ptr) { conj &= *ptr; }
	return conj;
}

#endif 
