#include "examples.h"
//---------------------------------------------------------------------------------------------
// calculates distance between phi and m by successive dilations
static unsigned distance(bdd phi, bdd m) {
	unsigned d=0;
	bdd p = phi;
	
	while((p&m)==bddfalse) {
		p = dilate(p);
		d++;
	}
	return d;
}
//---------------------------------------------------------------------------------------------
// negate variable if c is not one
#define nv(c,v) ( (c==1) ? (v) : (!(v)) )
//---------------------------------------------------------------------------------------------
void implementing_merging_ops_using_bdds_example(void) {
	std::cerr <<"Example from 'Implementing semantic merging operators using BDDs'.\n";
	propvars = 4;
	init();
	KSet kset(3);
	bdd a = bdd_ithvar(0);
	bdd b = bdd_ithvar(1);
	bdd c = bdd_ithvar(2);
	bdd d = bdd_ithvar(3);

	bdd IC  = (!a & !b & !c) | (!a & c & !d) | (a & !b & !c) | (a & b & c);
	kset[0] = (a & !b & c) | (a & b & !c & !d);
	kset[1] = (!a & !c & d) | (!a & !b & c & !d) | (a & c & !d);
	kset[2] = (!b & !c & d) | (!a & b & c & !d) | (a & b & c & d);
	
	for(unsigned i=0; i<2; i++) {
	for(unsigned j=0; j<2; j++) {
	for(unsigned k=0; k<2; k++) {
	for(unsigned l=0; l<2; l++) {
		bdd m = nv(i,a) & nv(j,b) & nv(k,c) & nv(l,d);
		
		//if((m&IC)==bddfalse) {
		//	continue;
		//}
	
		std::cerr << '(' << i << ',' << j << ',' << k << ',' << l << ')' << " & ";
		std::cerr << distance(kset[0],m) << " & ";
		std::cerr << distance(kset[1],m) << " & ";
		std::cerr << distance(kset[2],m) << " & ";

		std::cerr << '\n';
	
	}}}}

	bdd max = delta_max(kset, IC);
	bdd sigma = delta_sigma(kset, IC);
	bdd gmax = delta_G_max(kset, IC);
	bdd gmin = delta_G_min(kset, IC);
	
	std::cerr << "max=\n";
	bdd_allsat(max, allsatPrintHandler);
	std::cerr << std::endl;

	std::cerr << "sigma=\n";
	bdd_allsat(sigma, allsatPrintHandler);
	std::cerr << std::endl;
	
	std::cerr << "gmax=\n";
	bdd_allsat(gmax, allsatPrintHandler);
	std::cerr << std::endl;
	
	std::cerr << "gmin=\n";
	bdd_allsat(gmin, allsatPrintHandler);
	std::cerr << std::endl;

	cleanup();
}
//---------------------------------------------------------------------------------------------
void quota_and_gmin_merging_operators_example(void) {
	std::cerr <<"Example from 'Quota and Gmin merging operators'.\n";
	propvars = 3;
	init();
	
	KSet kset(4);
	
	bdd a = bdd_ithvar(0);
	bdd b = bdd_ithvar(1);
	bdd c = bdd_ithvar(2);

	bdd IC = a | !b;				// w \ {010,011}
	
	kset[0] = (a & !b) | (!a & !b & c);		// {100,001,101}
	kset[1] = !b & c;				// {001,101}
	kset[2] = !b & !c;				// {100,000}
	kset[3] = a & b & c;				// {111}

	bdd m = delta_G_min(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cout << std::endl;
	
	cleanup();
}
//---------------------------------------------------------------------------------------------
void merging_with_integrity_constraints_example(void) {
	std::cerr <<"Example from 'Merging with integrity constraints'.\n";
	propvars = 4;
	init();
	
	KSet kset(4);
	
	bdd S = bdd_ithvar(0);
	bdd T = bdd_ithvar(1);
	bdd P = bdd_ithvar(2);
	bdd I = bdd_ithvar(3);

	bdd IC = ( (S & T) | (S & P) | (T & P) ) >> I;	// any two builds will increase rent
	
	kset[0] = S & T & P;
	kset[1] = S & T & P;
	kset[2] = !S & !T & !P & !I;
	kset[3] = T & P & !I;

	std::cerr << "Sigma\n";
	bdd m = delta_sigma(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cerr << std::endl;
	std::cerr << bdd_satcount(m) << std::endl;
	
	std::cerr << "Gmax\n";
	m = delta_G_max(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cerr << std::endl;
	std::cerr << bdd_satcount(m) << std::endl;
	
	cleanup();
	std::cerr << '\n';
}
//---------------------------------------------------------------------------------------------
void on_the_logic_of_merging_example(void) {
	std::cerr <<"Example from 'On the logic of merging'.\n";
	propvars = 3;
	init();
	
	KSet kset(3);
	
	bdd S = bdd_ithvar(0);
	bdd D = bdd_ithvar(1);
	bdd O = bdd_ithvar(2);

	bdd IC = bddtrue;
	
	kset[0] = (S | O) & !D;
	kset[1] = (!S & D & !O) | (!S & !D & O);
	kset[2] = S & D & O;

	std::cerr << "Max\n";
	bdd m = delta_max(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cout << std::endl;
//	std::cout << bdd_satcount(m) << std::endl;
	
	std::cerr << "Sigma\n";
	m = delta_sigma(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cout << std::endl;
//	std::cout << bdd_satcount(m) << std::endl;
	
	std::cerr << "Gmax\n";
	m = delta_G_max(kset, IC);
	bdd_allsat(m, allsatPrintHandler);
	std::cout << std::endl;
//	std::cout << bdd_satcount(m) << std::endl;

	cleanup();
}
//---------------------------------------------------------------------------------------------
/*
// macros and gf are for findProfileWithDistinctMerges below
#define coin() ((bool)(genrand_real2()<0.25))
//---------------------------------------------------------------------------------------------
static bdd gf(void) {
	bdd a = bdd_ithvar(0);
	bdd b = bdd_ithvar(2);
	bdd c = bdd_ithvar(4);
	bdd d = bdd_ithvar(6);
	
	bdd disj = bddfalse;
	
	for(unsigned i=0; i<2; i++) {
	for(unsigned j=0; j<2; j++) {
	for(unsigned k=0; k<2; k++) {
	for(unsigned l=0; l<2; l++) {
		if(coin()) {
			disj |= nv(i,a) & nv(j,b) & nv(k,c) & nv(l,d);
		}
	}}}}

	return disj;
}
//---------------------------------------------------------------------------------------------
// brute force search for a profile that has different merges for all operators
void findProfileWithDistinctMerges(void) {
	std::cerr << "Making example\n";
	
	init_genrand(getSeed());
	
	bool found = false;
	propvars = 4;
	init();
	KSet kset(3);
	
	while(!found) {
		bdd IC = gf();
		kset[0] = gf();
		kset[1] = gf();
		kset[2] = gf();
		
		if((((IC & kset[0]) & kset[1]) & kset[2]) != bddfalse) {
			continue;
		}
	
		bdd max = delta_max(kset, IC);
		bdd sigma = delta_sigma(kset, IC);
		bdd gmax = delta_G_max(kset, IC);
		bdd gmin = delta_G_min(kset, IC);
		
		found = (max != sigma) && (max != gmax) && (max != gmin) && \
			(sigma != gmax) && (sigma != gmin) && \
			(gmax != gmin);
		

		if(found) {
			std::cerr << "IC=\n";
			bdd_allsat(IC, allsatPrintHandler);
			std::cerr << std::endl;
	
			std::cerr << "phi_1=\n";
			bdd_allsat(kset[0], allsatPrintHandler);
			std::cerr << std::endl;
			
			std::cerr << "phi_2=\n";
			bdd_allsat(kset[1], allsatPrintHandler);
			std::cerr << std::endl;
			
			std::cerr << "phi_3=\n";
			bdd_allsat(kset[2], allsatPrintHandler);
			std::cerr << std::endl;
		}
	}
	
	std::cerr <<"Result found.\n";

	cleanup();
}
*/
/*
Making example
IC=
+000X
+0010
+0110
+100X
+111X

phi_1=
+101X
+1100

phi_2=
+0001
+0010
+0101
+1X10

phi_3=
+0001
+0110
+1001
+1111

Result found.*/
//---------------------------------------------------------------------------------------------
