#!/usr/bin/python

import dircache
import os.path
import csv
import sys
import stats
import math

# indices in data files
P_MARKER=0
SEED=1
P_INCONSISTENT=2
P_PRODUCED=3
P_BDDNODES=4
P_DILATIONS=5	# N/A
P_CONJUNCTIONS=6 # N/A
P_MSECS=7
P_GBC=8

M_MARKER=9
M_SEED=10
M_OP=11
M_PRODUCED=12
M_BDDNODES=13
M_DILATIONS=14
M_CONJUNCTIONS=15
M_MSECS=16
M_GBC=17

# stats.median is a funny one
def mymed(l):
	t = l[:]
	t.sort()
	return t[len(t)/2]

# which data we want to process & graph
indices = [ P_PRODUCED, P_BDDNODES, P_MSECS, M_PRODUCED, M_BDDNODES, M_DILATIONS, M_CONJUNCTIONS, M_MSECS ]
#statOps = [ mymed ] #, stats.mean, stats.stdev ]
statOps = [ stats.mean, stats.stdev ]
directory = sys.argv[1]
output = []

for op in range(4):
	filename = os.path.join( directory, 'stats-%i.csv' % (op+1) )
	output.append( open( filename, 'w' ) )

for filename in dircache.listdir(directory):
	# skip no-data files - could be better
	if not filename[0].isdigit():
		continue
	
	fullname = os.path.join(directory, filename)
	v = int(fullname.split('-')[1])
	c = int(fullname.split('-')[2])
	op = int(fullname.split('-')[3].split('.')[0])-1
	R = float(c)/float(v)
	
	# read in file, skipping consistent profiles
	rows = [ row for row in csv.reader(open(fullname, 'r')) if int(row[P_INCONSISTENT])==1 ]
	if len(rows)==0:
		print 'ouch'
		continue
	
	data = [ row for row in rows if int(row[M_OP])==op+1 ]
	if len(data)==0:
		print filename, op
		continue

	# transpose
	data = zip(*data)
	
	# warn if gbcs have occured
	#if len([ r for r in data[P_GBC] if int(r)>0 ])>0:
	#	print "Garbage collection has occured when constructing formulae."
	
	## warn if gbcs have occured
	#if len([ r for r in data[M_GBC] if int(r)>0 ])>0:
	#	print "Garbage collection has occured when merging formulae."
	
	output[op].write('%i,' % v)
	
	for s in range(len(statOps)):
		# for each indice in indices, convert from strings to numbers and calculate the stats of that column
		statsRow = [ statOps[s] ( [ int(element) for element in data[i] ] ) for i in indices ]
		
		# convert to strings, then join with commas, then write into file
		output[op].write( ','.join( [ str(element) for element in statsRow ] ) )
		
		if s!=len(statOps)-1:
			output[op].write(',')
		
	
	output[op].write('\n')
		
for f in output:
	f.close()

