/*
IN ORDER TO GET FULL CACHE STATS FROM BUDDY, USE ENABLE-CACHE-STATS IN CONFIGURE
AND ADD #INCLUDE "CONFIG.H" IN KERNEL.H
*/

#include <iostream>
#include <string>
// for parse_into
#include <sstream>
// for getopt
#include <unistd.h>

// for ops
#include "operators/deltas.h"
// for timeout
#include "benchmark.h"

#include "randmerge.h"
#include "propformula.h"
#include "foformula.h"
#include "examples.h"
//-------------------------------------------------------------------------------------------
template<class T> bool parse_into(const std::string& s, T & v) {
	std::istringstream i(s);
	if (!(i >> v))
		return false;
	return true;
}
#define PARSE(s,v) assert(parse_into(std::string(s),(v)))
//-------------------------------------------------------------------------------------------
void usage(char **argv) {
	std::cerr << "Usage:\n";
	std::cerr << argv[0] << " -E\n\tRun examples from papers (default).\n";
	std::cerr << argv[0] << " -R [-o O] [-k K] [-p P] [-n N] [-c C] [-s S] [-t T]\n\tRun random tests a la 3SAT.\n";
	std::cerr << argv[0] << " -P [-o O] [-k K] [-p P]\n\tRun on propositional profile provided on stdin.\n";
	std::cerr << argv[0] << " -F [-o O] [-k K] [-r R] [-d D] [-a A]\n\tRun on first-order profile provided on stdin.\n";
	std::cerr << "where:\n";
	std::cerr << "\tO\tThe operator to use (1=max, 2=sigma, 3=gmax, 4=gmin).\n";
	std::cerr << "\tP\tNumber of propositional letters in the language.\n";
	std::cerr << "\tK\tNumber of formulae in the profile.\n";
	std::cerr << "\tN\tNumber of random tests to run.\n";
	std::cerr << "\tC\tNumber of clauses per formula in the profile.\n";
	std::cerr << "\tS\tInitial seed for random profiles (default: current time).\n";
	std::cerr << "\tT\tTimeout in seconds for random tests.\n";
	std::cerr << "\tR\tNumber of binary predicates.\n";
	std::cerr << "\tD\tDomain size of first-order structures.\n";
	std::cerr << "\tA\tArity of first-order predicates.\n";
}
//-------------------------------------------------------------------------------------------
int main(int argc, char ** argv) {
	// enter reasonable defaults before option parsing
	unsigned kbs = 3;
	unsigned clausesPerKB = 30;
	unsigned seed = getSeed();
	unsigned repetitions = 1;
	int op = 1;
	int c;
	enum {EXAMPLES, RANDOM, PROP_INPUT, FO_INPUT } mode = EXAMPLES; 
	
	propvars = 10;
	opterr = 0;
	
	while ((c = getopt (argc, argv, "o:p:n:k:c:s:t:r:d:a:ERPFh")) != -1) {
		switch (c) {
			case 'o':
				PARSE(optarg, op);
				op--;
				break;
			case 'p':
				PARSE(optarg, propvars);
				break;
			case 'n':
				PARSE(optarg, repetitions);
				break;
			case 'k':
				PARSE(optarg, kbs);
				break;
			case 'c':
				PARSE(optarg, clausesPerKB);
				break;
			case 's':
				PARSE(optarg, seed);
				break;
			case 't':
				PARSE(optarg, timeout);
				break;
			case 'r':
				PARSE(optarg, signature.predicates);
				break;
			case 'd':
				PARSE(optarg, signature.dsize);
				break;
			case 'a':
				PARSE(optarg, signature.arity);
				break;
			case 'E':
				mode = EXAMPLES;
				break;
			case 'R':
				mode = RANDOM;
				break;
			case 'P':
				mode = PROP_INPUT;
				break;
			case 'F':
				mode = FO_INPUT;
				break;
			case 'h':
				usage(argv);
				return 0;
			default:
				std::cerr << "Cannot parse command line.\n";
				usage(argv);
				return 1;
		}
	}

	switch(mode) {
		case EXAMPLES:
			on_the_logic_of_merging_example();
			merging_with_integrity_constraints_example();
			quota_and_gmin_merging_operators_example();
			implementing_merging_ops_using_bdds_example();
			return 0;
		case RANDOM:
			assert(op>=0 && op<4);
			assert(propvars > 0 && propvars < 1000000);
			assert(repetitions > 0 && repetitions < 1000000);
			assert(kbs > 0 && kbs < 1000);
			assert(clausesPerKB > 0 && clausesPerKB < 1000);
			assert(timeout>0 && timeout<3600);
			run_random(kbs, clausesPerKB, seed, repetitions, op);
			return 0;
		case PROP_INPUT:
			assert(op>=0 && op<4);
			assert(propvars > 0 && propvars < 1000000);
			assert(kbs > 0 && kbs < 1000);
			run_prop_input(kbs, op);
			return 0;
		case FO_INPUT:
			assert(op>=0 && op<4);
			assert(kbs > 0 && kbs < 1000);
			assert(signature.predicates>0 && signature.predicates<100);
			assert(signature.dsize>0 && signature.dsize<10);
			assert(signature.arity>0 && signature.arity<10);
			signature.first_order = true;
			run_fo_input(kbs, op);
			return 1;
		default:
			return 1;
	}
	
	return 1;
}
//-------------------------------------------------------------------------------------------

