#include "deltas.h"

#include <utility>
#include <set>

#define DVEC 2

typedef std::pair<unsigned,unsigned> Point;

#if DVEC==1
typedef std::vector<Point> DVec;

inline bdd dvec_conj(const DVec &dv, const KSetSet &dilation, const bdd &IC) {
	bdd conj=IC;
	for(DVec::const_iterator ptr=dv.begin(); conj!=bddfalse && ptr!=dv.end(); ++ptr) {
		unsigned i=ptr->first;
		unsigned j=ptr->second;
		conj &= dilation.at(i).at(j);
	}
	return conj;
}

#define C(V) (dvec_conj(V, dilation, IC))
#endif

#if DVEC==2
class DVec: public std::vector<Point> {
	public:
		DVec(const KSetSet & d, const bdd & IC) : dilation(d), conjunction(IC) {}
		
		DVec(const DVec & dv) : std::vector<Point>(dv), dilation(dv.dilation), conjunction(dv.conjunction) {}
		
		void push_back(const Point & p) {
			std::vector<Point>::push_back(p);
			conjunction &= dilation.at(p.first).at(p.second);
		}
		
		friend bdd C(const DVec &);
		
	private:
		const KSetSet & dilation;
		bdd conjunction;
};

bdd C(const DVec & dv) {
	return dv.conjunction;
}
#endif

#if DVEC==1 || DVEC==2
inline bool notin(const DVec & dv, unsigned j) {
	for(DVec::const_iterator ptr=dv.begin(); ptr!=dv.end(); ++ptr) {
		if(ptr->first==j) {
			return false;
		}
	}
	return true;
}

inline bool in(const DVec & dv, unsigned j) {
	return !notin(dv, j);
}
#endif

typedef std::set<DVec> DSet;

inline bool allfalse(const std::vector<bool> &v) {
	for(std::vector<bool>::const_iterator p=v.begin(); p!=v.end(); ++p) {
		if(*p==true) {
			return false;
		}
	}
	return true;
}

bdd delta_G_min(const KSet &kset, const bdd &IC) {
	// 2-dimensional array keeping all dilations
	KSetSet dilation(kset.size());
	for(unsigned i=0; i<dilation.size(); i++) {
		// reserve space for up to propvars number of dilations for each formula
		dilation.at(i).reserve(propvars);
		// and initialise with the members of kset
		dilation.at(i).push_back( kset.at(i) );
	}

	DSet * S = new DSet;
	// insert empty distance vector
#if DVEC==1
	S->insert(DVec());
#endif

#if DVEC==2
	S->insert(DVec(dilation, IC));
#endif

	// using property that for any V in S, |V|=max{ |Y| for Y in S }
	for(unsigned d=0; !TERMINATE() && S->begin()->size() < kset.size(); ++d) {
/*		std::cerr << "====\n";
		for(DSet::const_iterator v=S->begin(); v!=S->end(); ++v) {
			for(DVec::const_iterator p=v->begin(); p!=v->end(); ++p) {
				std::cerr << '(' << p->first << ',' << p->second << ')' << ',';
				//std::cerr << *p << ',';
			}
			std::cerr << '\n';
		}
		std::cerr << "----\n";*/
		
		
		if(1<=d && d<=propvars) {
			for(KSetSet::iterator dptr=dilation.begin(); dptr!=dilation.end(); ++dptr) {
				dptr->push_back( dilate( *(dptr->end()-1) ) );
			}
			// increase global var for benchmarking
			dilations++;
		}
		
		DSet *Sp = new DSet;
#if DVEC==1
		Sp->insert(DVec());
#endif
#if DVEC==2
		Sp->insert(DVec(dilation, IC));
#endif
		for(DSet::const_iterator v=S->begin(); v!=S->end(); ++v) {
			std::vector<bool> counter(kset.size(), false);
			
			do {
				for(std::vector<bool>::iterator p=counter.begin(); p!=counter.end(); ++p) {
					if(*p==false) {
						*p = true;
						break;
					} else {
						*p = false;
					}
				}
				
				DVec vp(*v);
				bool skip=false;
				for(unsigned i=0; i<kset.size(); ++i) {
					if(counter[i]) {
						conjunctions++;
						
						if(in(vp, i) || (C(vp) & dilation.at(i).at(d)) == bddfalse) {
							skip = true;
							break;
						}
						
						vp.push_back(Point(i,d));
					}
				}
				
				if(skip) {
					continue;
				}
				
				if(vp.size() > v->size()) {
					if (vp.size() > Sp->begin()->size()) {
						Sp->clear();
						Sp->insert(vp);
					} else if (vp.size() == Sp->begin()->size()) {
						Sp->insert(vp);
					}
				}
			} while( !allfalse(counter) );
		}
		
		if(Sp->size()>0 && Sp->begin()->size()>S->begin()->size()) {
			delete S;
			S = Sp;
		} else {
			delete Sp;
		}
		Sp = NULL;
		
	}
	
	bdd solution = bddfalse;
	for(DSet::const_iterator v=S->begin(); v!=S->end(); ++v) {
/*		for(DVec::const_iterator p=v->begin(); p!=v->end(); ++p) {
			std::cerr << '(' << p->first << ',' << p->second << ')' << ',';
			std::cerr << '(' << p->first << ',' << p->second << ')' << ',';
		}
		std::cerr << '\n';*/
		solution |= C(*v);
	}
/*	std::cerr << "======================================\n";*/
	
	delete S;
	return solution;
}
