#include "operators/deltas.h"

#include "util/composition-colex.h"

bdd delta_sigma(const KSet &kset, const bdd &IC) {
	// check the trivial condition
	{
		bdd conj = conjunction(kset, IC);
		if(conj != bddfalse) {
			return conj;
		}
	}
	
	// 2-dimensional array keeping all dilations
	std::vector<KSet> dilation(kset.size());
	for(unsigned i=0; i<dilation.size(); i++) {
		// reserve space for up to propvars number of dilations for each formula
		dilation.at(i).reserve(propvars);
		// and initialise with the members of kset
		dilation.at(i).push_back(kset.at(i));
	}

	bdd solution = bddfalse;
	for(unsigned d=1; !TERMINATE() && solution==bddfalse; d++) {
		// dilate the top formulas
		if(d<=propvars) {
			for(unsigned i=0; i<dilation.size(); i++) {
				dilation.at(i).push_back( dilate( *(dilation.at(i).end()-1) ) );
			}
			// increase global var for benchmarking
			dilations++;
		}
		
		// create enumerator of compositions of the number d
		composition_colex comp(d, kset.size());
		do {
			// get vector from de-composition object
			const unsigned long * v = comp.data();
		
			// check that this is a valid distance vector
			if(*std::max_element(v, v+kset.size())>propvars) {
				continue;
			}
			
			// calculate conjunction with ICs
			bdd conj = IC;
			for(unsigned i=0; i<dilation.size() && conj!=bddfalse; i++) {
				conj &= dilation.at(i).at(v[i]);
			}
			conjunctions++;

			// keep solution by or-ing with previously found
			if(conj!=bddfalse) {
				solution |= conj;
			}
		} while( !TERMINATE() && comp.next() );
	}
		
	return solution;
}


/*
bdd delta_sigma(const KSet &kset, const bdd &IC) {
	// check the trivial condition
	{
		bdd conj = conjunction(kset, IC);
		if(conj != bddfalse) {
			return conj;
		}
	}
	
	
	// 2-dimensional array keeping all dilations
	std::vector<KSet> dilation(kset.size());
	for(unsigned i=0; i<dilation.size(); i++) {
		// reserve space for up to propvars number of dilations for each formula
		dilation.at(i).reserve(propvars);
		// and initialise with the members of kset
		dilation.at(i).push_back(kset.at(i));
	}

	bdd solution = bddtrue;
	unsigned bestBound = propvars * kset.size();
	
	std::set< IVec > vset;

	// only when the number of dilations grows beyond the best sum can we stop
	// think of (bestBound, 0, ..., 0)
	for(unsigned j=1; !TERMINATE() && j<=bestBound; j++) {
		if(j<=propvars) {
			// dilate the top formulas
			for(unsigned i=0; i<dilation.size(); i++) {
				dilation.at(i).push_back( dilate( *(dilation.at(i).end()-1) ) );
			}
			// increase global var for benchmarking
			dilations++;
		}
		
		// every time we dilate, we add 1 * kset.size() to the maximum sum of distances
		// so we can search between j and j*kset.size() inclusive for sums
		// sum<=bestBound will stop the loop when something strictly better than the next sum has been found
		for(unsigned sum=j; !TERMINATE() && sum<=j*kset.size() && sum<=bestBound; sum++) {
			
			composition_colex comp(sum, kset.size());
			
			do {
				// get vector from de-composition object
				const unsigned long * v = comp.data();
				
				// we may enumerate vectors that are out of bounds
				// skip those (they will be enumerated again later)
				unsigned k;
				for(k=0; k<dilation.size() && v[k]<dilation.at(k).size(); k++) {}
				if(k!=dilation.size()) {
					continue;
				}

				IVec d(v, v+kset.size());
				if(vset.find(d)!=vset.end()) {
					continue;
				}
				vset.insert(d);

				// calculate conjunction with ICs
				bdd conj = IC;
				for(unsigned i=0; i<dilation.size() && conj!=bddfalse; i++) {
					conj &= dilation.at(i).at(v[i]);
				}
				
				if(conj==bddfalse) {
					continue;
				}
				
				if(sum<bestBound) {
					solution = conj;
					bestBound = sum;
				} else if(sum==bestBound) {
					solution |= conj;
				}
			} while( !TERMINATE() && comp.next() );
		}
	}
		
	return solution;
}

*/
